﻿;Screenfilter-Module

DeclareModule Screenfilter
  EnableExplicit
  
  UsePNGImageDecoder()
  UseJPEGImageDecoder()
  
  CompilerSelect #PB_Compiler_OS
    CompilerCase #PB_OS_Linux
      Import ""
        glXGetProcAddress(s.p-ascii) As "glXGetProcAddress"
      EndImport
    CompilerCase #PB_OS_Windows
      CompilerIf Subsystem("OpenGL") = #False
        MessageRequester("Error", "Setze das Subsystem zu OpenGL: Compiler... Compiler Options")
        End
      CompilerEndIf
      CompilerIf (#PB_Compiler_Processor = #PB_Processor_x86)
        Import "opengl32.lib"
          wglGetProcAddress(s.p-ascii) As "_wglGetProcAddress@4"
        EndImport
      CompilerElse   
        Import "opengl32.lib"
          wglGetProcAddress(s.p-ascii) As "wglGetProcAddress"
        EndImport
      CompilerEndIf
    CompilerDefault
      CompilerError "Sie benutzen ein nicht unterstützes Betriebssystem."
  CompilerEndSelect
  
  Structure System
    Shader_Width.i
    Shader_Height.i
    MouseX.i
    MouseY.i
    
    App_CurrentTime.i
    App_StartTime.i
    
    Shader_Vertex_Text.s
    Shader_Fragment_Text.s
    Shader_Vertex.i
    Shader_Fragment.i
    Shader_Program.i
    
    Shader_Uniform_Time.i
    Shader_Uniform_Resolution.i
    Shader_Uniform_Mouse.i
    Shader_Uniform_SurfacePosition.i
    
    Shader_Image.i
  EndStructure
  
  #GL_BGR = $80E0
  #GL_BGRA = $80E1
  
  #GL_VERTEX_SHADER = $8B31
  #GL_FRAGMENT_SHADER = $8B30
  
  Global System.System
  
  Global DebugInfos.b = #False
  
  Prototype glCreateShader(type.l)
  Prototype glCreateProgram()
  Prototype glCompileShader(shader.l)
  Prototype glLinkProgram(shader.l)
  Prototype glUseProgram(shader.l)
  Prototype glAttachShader(Program.l, shader.l)
  Prototype glShaderSource(shader.l, numOfStrings.i, *strings, *lenOfStrings)
  Prototype.i glGetUniformLocation(Program.l, name.p-ascii)
  Prototype glUniform1i(location.i, v0.i)
  Prototype glUniform2i(location.i, v0.i, v1.i)
  Prototype glUniform1f(location.i, v0.f)
  Prototype glUniform2f(location.i, v0.f, v1.f)
  Prototype glGetShaderInfoLog(shader.l, bufSize.l, *length_l, *infoLog)
  
  Prototype glDeleteShader(shader.l)
  Prototype glDeleteProgram(Program.l)
  
  Prototype.i glGetProcAddress(name.p-ascii)
  
  Global glGetProcAddress.glGetProcAddress
  Global glCreateShader.glCreateShader
  Global glCreateProgram.glCreateProgram
  Global glCompileShader.glCompileShader
  Global glLinkProgram.glLinkProgram
  Global glUseProgram.glUseProgram
  Global glAttachShader.glAttachShader
  Global glShaderSource.glShaderSource
  Global glGetUniformLocation.glGetUniformLocation
  Global glUniform1i.glUniform1i
  Global glUniform2i.glUniform2i
  Global glUniform1f.glUniform1f
  Global glUniform2f.glUniform2f
  Global glGetShaderInfoLog.glGetShaderInfoLog
  
  Global glDeleteShader.glDeleteShader
  Global glDeleteProgram.glDeleteProgram
  
  Declare.i Get_ScreenTexture(ImageHandle.i)
  Declare Set_ScreenFilter(Fragment.i, TextureID.i)
  Declare Render_ScreenFilter(w.i, h.i, ftime.i)
  Declare Init_FilterModule()
  Declare Show_DebugInfos(status.b)
  Declare Disable_Filter()
  Declare Delete_ScreenTexture(TextureHandle)
EndDeclareModule

Module Screenfilter
  EnableExplicit
  
  Procedure Init_FilterModule()
    
    ;     glMatrixMode_(#GL_PROJECTION)
    ;     glLoadIdentity_()
    ;     glOrtho_(0, 1.0, 0, 1.0, -10.0, 10.0)
    ;     
    ;     glMatrixMode_(#GL_MODELVIEW)
    ;     glLoadIdentity_()
    
    System\App_StartTime = ElapsedMilliseconds()
    
  EndProcedure
  
  Procedure Show_DebugInfos(status.b)
    If status = #True
      DebugInfos = #True
    ElseIf status = #False
      DebugInfos = #False
    EndIf
  EndProcedure
  
  Procedure Disable_Filter()
    If System\Shader_Program <> 0 ;Sollte den vorherigen Shader löschen.
      glUseProgram(0)
      glDeleteProgram(System\Shader_Program)
      ProcedureReturn #False
    Else
      ProcedureReturn #True
    EndIf
  EndProcedure
  
  Procedure.i Get_ScreenTexture(ImageHandle.i)
    
    Define.i ImageW, ImageH, ImageD
    Define.i MemoryAddress
    Define.i TextureHandle
    
    If IsImage(ImageHandle) = 0
      ProcedureReturn #False
    EndIf
    
    ImageD = ImageDepth(ImageHandle, #PB_Image_InternalDepth)
    
    StartDrawing(ImageOutput(ImageHandle))
    MemoryAddress = DrawingBuffer()
    StopDrawing()
    
    If MemoryAddress = 0
      ProcedureReturn #False
    EndIf
    
    glEnable_(#GL_TEXTURE_2D)
    
    glGenTextures_(1, @TextureHandle)
    glBindTexture_(#GL_TEXTURE_2D, TextureHandle)
    
    ImageW = ImageWidth(ImageHandle)
    ImageH = ImageHeight(ImageHandle)
    
    CompilerSelect #PB_Compiler_OS
      CompilerCase #PB_OS_Linux
        If ImageD = 32
          glTexImage2D_(#GL_TEXTURE_2D, 0, #GL_RGBA, ImageW, ImageH, 0, #GL_RGBA, #GL_UNSIGNED_BYTE, MemoryAddress)
        Else
          glTexImage2D_(#GL_TEXTURE_2D, 0, #GL_RGB, ImageW, ImageH, 0, #GL_RGB, #GL_UNSIGNED_BYTE, MemoryAddress)
        EndIf
      CompilerCase #PB_OS_Windows
        If ImageD = 32
          glTexImage2D_(#GL_TEXTURE_2D, 0, 4, ImageW, ImageH, 0, #GL_BGRA, #GL_UNSIGNED_BYTE, MemoryAddress)
        Else
          glTexImage2D_(#GL_TEXTURE_2D, 0, 3, ImageW, ImageH, 0, #GL_BGR, #GL_UNSIGNED_BYTE, MemoryAddress)
        EndIf
    CompilerEndSelect
    
    glTexParameteri_(#GL_TEXTURE_2D, #GL_TEXTURE_WRAP_S, #GL_REPEAT)
    glTexParameteri_(#GL_TEXTURE_2D, #GL_TEXTURE_WRAP_T, #GL_REPEAT)
    glTexParameteri_(#GL_TEXTURE_2D, #GL_TEXTURE_MIN_FILTER, #GL_NEAREST)
    glTexParameteri_(#GL_TEXTURE_2D, #GL_TEXTURE_MAG_FILTER, #GL_NEAREST)
    
    glDisable_(#GL_TEXTURE_2D)
    
    ProcedureReturn TextureHandle
    
  EndProcedure
  
  Procedure Delete_ScreenTexture(TextureHandle)
    Protected ok.b
    If glAreTexturesResident_(1, @TextureHandle, @ok);
      glDeleteTextures_(1, @TextureHandle)
      ProcedureReturn #True
    Else
      ProcedureReturn #False
    EndIf
  EndProcedure
  
  Procedure Load_ShaderFunctions()
    CompilerSelect #PB_Compiler_OS
      CompilerCase #PB_OS_Linux
        glGetProcAddress = @glXGetProcAddress()
      CompilerCase #PB_OS_Windows
        glGetProcAddress = @wglGetProcAddress()
      CompilerDefault
        CompilerError "Sie benutzen ein nicht unterstützes Betriebssystem."
    CompilerEndSelect
    
    glCreateShader = glGetProcAddress("glCreateShader")
    glCreateProgram = glGetProcAddress("glCreateProgram")
    glCompileShader = glGetProcAddress("glCompileShader")
    glLinkProgram = glGetProcAddress("glLinkProgram")
    glUseProgram = glGetProcAddress("glUseProgram")
    glAttachShader = glGetProcAddress("glAttachShader")
    glShaderSource = glGetProcAddress("glShaderSource")
    glGetUniformLocation = glGetProcAddress("glGetUniformLocation")
    glUniform1i = glGetProcAddress("glUniform1i")
    glUniform2i = glGetProcAddress("glUniform2i")
    glUniform1f = glGetProcAddress("glUniform1f")
    glUniform2f = glGetProcAddress("glUniform2f")
    glGetShaderInfoLog = glGetProcAddress("glGetShaderInfoLog")
    
    glDeleteShader = glGetProcAddress("glDeleteShader")
    glDeleteProgram = glGetProcAddress("glDeleteProgram")
  EndProcedure
  
  Procedure Shader_Compile_Link_Use(Vertex.s,Fragment.s,Use.i=1)
    Protected *TxtPointer, VertShader.i, FragShader.i, Program.i
    Protected Textlength.i, ShaderScript.s = Space(1024)
    
    Load_ShaderFunctions()
    
    ;Compile Vertex shader
    VertShader.i = glCreateShader(#GL_VERTEX_SHADER)
    *TxtPointer = Ascii(Vertex)
    glShaderSource(VertShader, 1, @*TxtPointer, #Null)
    glCompileShader(VertShader)
    
    If DebugInfos = #True : Debug "Vert: "+VertShader : EndIf
    
    glGetShaderInfoLog(VertShader,1023,@Textlength,@ShaderScript)
    
    If DebugInfos = #True : Debug PeekS(@ShaderScript, -1, #PB_Ascii) : EndIf
    
    ;Compile Fragment Shader
    FragShader.i = glCreateShader(#GL_FRAGMENT_SHADER)
    *TxtPointer = Ascii(Fragment)
    glShaderSource(FragShader, 1, @*TxtPointer, #Null)
    glCompileShader(FragShader)
    
    If DebugInfos = #True : Debug "Frag: "+FragShader : EndIf
    
    glGetShaderInfoLog(FragShader, 1023, @Textlength, @ShaderScript)
    
    If DebugInfos = #True : Debug PeekS(@ShaderScript, -1, #PB_Ascii) : EndIf
    
    ;Create Shader Program
    Program = glCreateProgram()
    
    If DebugInfos = #True : Debug "Shader: "+Program : EndIf
    
    glAttachShader(Program,VertShader)
    glAttachShader(Program,FragShader)
    
    glDeleteShader(System\Shader_Vertex)
    glDeleteShader(System\Shader_Fragment)
    
    glLinkProgram(Program)
    
    glDeleteProgram(System\Shader_Program)
    
    If Use = 1
      glUseProgram(Program)
    EndIf
    
    ProcedureReturn Program
  EndProcedure
  
  Procedure Set_ScreenFilter(Fragment.i, TextureID.i)
    
    If Fragment < 0
      ProcedureReturn #False
    EndIf
    
    If System\Shader_Program <> 0 ;Sollte den vorherigen Shader löschen.
      glUseProgram(0)
    EndIf
    
    If Fragment >= 0 And Fragment < 5
      
      System\Shader_Vertex_Text = "#version 130"+#CRLF$ ;Version ist Pflicht!
      System\Shader_Vertex_Text + "attribute vec2 surfacePosAttrib;"
      System\Shader_Vertex_Text + "   void main() {"+#CRLF$
      System\Shader_Vertex_Text + "      gl_Position = ftransform();"+#CRLF$ ;ftranform() ist wie ein bisschen Magie
      System\Shader_Vertex_Text + "   }"+#CRLF$
      
      CompilerSelect #PB_Compiler_OS
        CompilerCase #PB_OS_Linux
          Select Fragment
            Case 0
              System\Shader_Fragment_Text = "#version 120"+#CRLF$
              System\Shader_Fragment_Text + "//Normal-Image (Linux-Version)"+#CRLF$
              System\Shader_Fragment_Text + "uniform sampler2D color_texture;"+#CRLF$
              System\Shader_Fragment_Text + "uniform vec2 resolution;"+#CRLF$
              System\Shader_Fragment_Text + "void main() {"+#CRLF$
              System\Shader_Fragment_Text + "   vec2 pos = gl_FragCoord.xy/resolution;"+#CRLF$
              System\Shader_Fragment_Text + "   pos.x = 1.0 - pos.x;"+#CRLF$
              System\Shader_Fragment_Text + "   vec4 color = texture2D(color_texture, -pos);"+#CRLF$
              System\Shader_Fragment_Text + "   gl_FragColor = color;"+#CRLF$
              System\Shader_Fragment_Text + "}"+#CRLF$
            Case 1
              System\Shader_Fragment_Text = "#version 120"+#CRLF$
              System\Shader_Fragment_Text + "//Gray-Image (Linux-Version)"+#CRLF$
              System\Shader_Fragment_Text + "uniform sampler2D color_texture;"+#CRLF$
              System\Shader_Fragment_Text + "uniform vec2 resolution;"+#CRLF$
              System\Shader_Fragment_Text + "void main() {"+#CRLF$
              System\Shader_Fragment_Text + "   vec2 pos = gl_FragCoord.xy/resolution;"+#CRLF$
              System\Shader_Fragment_Text + "   pos.x = 1.0 - pos.x;"+#CRLF$
              System\Shader_Fragment_Text + "   vec4 color = texture2D(color_texture, -pos);"+#CRLF$
              System\Shader_Fragment_Text + "   float grey = (0.2126 * color.r) + (0.7152 * color.g) + (0.0722 * color.b);"+#CRLF$
              System\Shader_Fragment_Text + "   color.rgb += (grey - color.rgb);"+#CRLF$
              System\Shader_Fragment_Text + "   gl_FragColor = color;"+#CRLF$
              System\Shader_Fragment_Text + "}"+#CRLF$
            Case 2
              System\Shader_Fragment_Text = "#version 130"+#CRLF$
              System\Shader_Fragment_Text + "//Sepia-Image (Linux-Version)"+#CRLF$
              System\Shader_Fragment_Text + "uniform sampler2D color_texture;"+#CRLF$
              System\Shader_Fragment_Text + "uniform vec2 resolution;"+#CRLF$
              System\Shader_Fragment_Text + "void main() {"+#CRLF$
              System\Shader_Fragment_Text + "   float amount = 1.0;"+#CRLF$
              System\Shader_Fragment_Text + "   vec2 pos = gl_FragCoord.xy/resolution;"+#CRLF$
              System\Shader_Fragment_Text + "   pos.x = 1.0 - pos.x;"+#CRLF$
              System\Shader_Fragment_Text + "   vec4 color = texture2D(color_texture, -pos);"+#CRLF$
              System\Shader_Fragment_Text + "   float r = color.r;"+#CRLF$
              System\Shader_Fragment_Text + "   float g = color.g;"+#CRLF$
              System\Shader_Fragment_Text + "   float b = color.b;"+#CRLF$
              System\Shader_Fragment_Text + "   color.r = min(1.0, (r * (1.0 - (0.607 * amount))) + (g * (0.769 * amount)) + (b * (0.189 * amount)));"+#CRLF$
              System\Shader_Fragment_Text + "   color.g = min(1.0, (r * 0.349 * amount) + (g * (1.0 - (0.314 * amount))) + (b * 0.168 * amount));"+#CRLF$
              System\Shader_Fragment_Text + "   color.b = min(1.0, (r * 0.272 * amount) + (g * 0.534 * amount) + (b * (1.0 - (0.869 * amount))));"+#CRLF$
              System\Shader_Fragment_Text + "   gl_FragColor = color;"+#CRLF$
              System\Shader_Fragment_Text + "}"+#CRLF$
            Case 3
              System\Shader_Fragment_Text = "#version 130"+#CRLF$
              System\Shader_Fragment_Text + "//Invert-Image (Linux-Version)"+#CRLF$
              System\Shader_Fragment_Text + "uniform sampler2D color_texture;"+#CRLF$
              System\Shader_Fragment_Text + "uniform vec2 resolution;"+#CRLF$
              System\Shader_Fragment_Text + "void main() {"+#CRLF$
              System\Shader_Fragment_Text + "   float amount = 1.0;"+#CRLF$
              System\Shader_Fragment_Text + "   vec2 pos = gl_FragCoord.xy/resolution;"+#CRLF$
              System\Shader_Fragment_Text + "   pos.x = 1.0 - pos.x;"+#CRLF$
              System\Shader_Fragment_Text + "   vec4 color = texture2D(color_texture, -pos);"+#CRLF$
              System\Shader_Fragment_Text + "   gl_FragColor = texture2D(color_texture, -pos);"+#CRLF$
              System\Shader_Fragment_Text + "   gl_FragColor.rgb = 1.0 - gl_FragColor.rgb;"+#CRLF$
              System\Shader_Fragment_Text + "}"+#CRLF$
            Case 4
              System\Shader_Fragment_Text = "#version 130"+#CRLF$
              System\Shader_Fragment_Text + "//Old-Movie (Linux-Version)"+#CRLF$
              System\Shader_Fragment_Text + "#define BLACK_AND_WHITE"+#CRLF$
              System\Shader_Fragment_Text + "#define LINES_AND_FLICKER"+#CRLF$
              System\Shader_Fragment_Text + "#define BLOTCHES"+#CRLF$
              System\Shader_Fragment_Text + "#define GRAIN"+#CRLF$
              System\Shader_Fragment_Text + "#define FREQUENCY 15.0"+#CRLF$
              System\Shader_Fragment_Text + "uniform sampler2D color_texture;"+#CRLF$
              System\Shader_Fragment_Text + "uniform vec2 resolution;"+#CRLF$
              System\Shader_Fragment_Text + "uniform float time;"+#CRLF$
              System\Shader_Fragment_Text + "vec2 uv;"+#CRLF$
              System\Shader_Fragment_Text + "float rand(vec2 co){"+#CRLF$
              System\Shader_Fragment_Text + "return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);"+#CRLF$
              System\Shader_Fragment_Text + "}"+#CRLF$
              System\Shader_Fragment_Text + "float rand(float c){"+#CRLF$
              System\Shader_Fragment_Text + "return rand(vec2(c,1.0));"+#CRLF$
              System\Shader_Fragment_Text + "}"+#CRLF$
              System\Shader_Fragment_Text + "float randomLine(float seed){"+#CRLF$
              System\Shader_Fragment_Text + "float b = 0.01 * rand(seed);"+#CRLF$
              System\Shader_Fragment_Text + "float a = rand(seed+1.0);"+#CRLF$
              System\Shader_Fragment_Text + "float c = rand(seed+2.0) - 0.5;"+#CRLF$
              System\Shader_Fragment_Text + "float mu = rand(seed+3.0);"+#CRLF$
              System\Shader_Fragment_Text + "float l = 1.0;"+#CRLF$
              System\Shader_Fragment_Text + "if ( mu > 0.2)"+#CRLF$
              System\Shader_Fragment_Text + "l = pow(  abs(a * uv.x + b * uv.y + c ), 1.0/8.0 );"+#CRLF$
              System\Shader_Fragment_Text + "else"+#CRLF$
              System\Shader_Fragment_Text + "l = 2.0 - pow( abs(a * uv.x + b * uv.y + c), 1.0/8.0 );"+#CRLF$
              System\Shader_Fragment_Text + "return mix(0.5, 1.0, l);"+#CRLF$
              System\Shader_Fragment_Text + "}"+#CRLF$
              System\Shader_Fragment_Text + "float randomBlotch(float seed){"+#CRLF$
              System\Shader_Fragment_Text + "float x = rand(seed);"+#CRLF$
              System\Shader_Fragment_Text + "float y = rand(seed+1.0);"+#CRLF$
              System\Shader_Fragment_Text + "float s = 0.01 * rand(seed+2.0);"+#CRLF$
              System\Shader_Fragment_Text + "vec2 p = vec2(x,y) - uv;"+#CRLF$
              System\Shader_Fragment_Text + "p.x *= resolution.x / resolution.y;"+#CRLF$
              System\Shader_Fragment_Text + "float a = atan(p.y,p.x);"+#CRLF$
              System\Shader_Fragment_Text + "float v = 1.0;"+#CRLF$
              System\Shader_Fragment_Text + "float ss = s*s * (sin(6.2831*a*x)*0.1 + 1.0);"+#CRLF$
              System\Shader_Fragment_Text + "if ( dot(p,p) < ss ) v = 0.2;"+#CRLF$
              System\Shader_Fragment_Text + "else"+#CRLF$
              System\Shader_Fragment_Text + "v = pow(dot(p,p) - ss, 1.0/16.0);"+#CRLF$
              System\Shader_Fragment_Text + "return mix(0.3 + 0.2 * (1.0 - (s / 0.02)), 1.0, v);"+#CRLF$
              System\Shader_Fragment_Text + "}"+#CRLF$
              System\Shader_Fragment_Text + "void main(void){"+#CRLF$
              System\Shader_Fragment_Text + "uv = gl_FragCoord.xy / resolution.xy;"+#CRLF$
              System\Shader_Fragment_Text + "uv.x = 1.0 - uv.x;"+#CRLF$
              System\Shader_Fragment_Text + "float t = float(int(time * FREQUENCY));"+#CRLF$
              System\Shader_Fragment_Text + "vec2 suv = uv + 0.002 * vec2( rand(t), rand(t + 23.0));"+#CRLF$
              System\Shader_Fragment_Text + "vec3 image = texture2D( color_texture, -vec2(suv.x, suv.y) ).xyz;"+#CRLF$
              System\Shader_Fragment_Text + "#ifdef BLACK_AND_WHITE"+#CRLF$
              System\Shader_Fragment_Text + "float luma = dot( vec3(0.2126, 0.7152, 0.0722), image );"+#CRLF$
              System\Shader_Fragment_Text + "vec3 oldImage = luma * vec3(0.7, 0.7, 0.7);"+#CRLF$
              System\Shader_Fragment_Text + "#else"+#CRLF$
              System\Shader_Fragment_Text + "vec3 oldImage = image;"+#CRLF$
              System\Shader_Fragment_Text + "#endif"+#CRLF$
              System\Shader_Fragment_Text + "float vI = 16.0 * (uv.x * (1.0-uv.x) * uv.y * (1.0-uv.y));"+#CRLF$
              System\Shader_Fragment_Text + "vI *= mix( 0.7, 1.0, rand(t + 0.5));"+#CRLF$
              System\Shader_Fragment_Text + "vI += 1.0 + 0.4 * rand(t+8.);"+#CRLF$
              System\Shader_Fragment_Text + "vI *= pow(16.0 * uv.x * (1.0-uv.x) * uv.y * (1.0-uv.y), 0.4);"+#CRLF$
              System\Shader_Fragment_Text + "#ifdef LINES_AND_FLICKER"+#CRLF$
              System\Shader_Fragment_Text + "int l = int(8.0 * rand(t+7.0));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 0 < l ) vI *= randomLine( t+6.0+17.* float(0));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 1 < l ) vI *= randomLine( t+6.0+17.* float(1));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 2 < l ) vI *= randomLine( t+6.0+17.* float(2));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 3 < l ) vI *= randomLine( t+6.0+17.* float(3));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 4 < l ) vI *= randomLine( t+6.0+17.* float(4));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 5 < l ) vI *= randomLine( t+6.0+17.* float(5));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 6 < l ) vI *= randomLine( t+6.0+17.* float(6));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 7 < l ) vI *= randomLine( t+6.0+17.* float(7));"+#CRLF$
              System\Shader_Fragment_Text + "#endif"+#CRLF$
              System\Shader_Fragment_Text + "#ifdef BLOTCHES"+#CRLF$
              System\Shader_Fragment_Text + "int s = int( max(8.0 * rand(t+18.0) -2.0, 0.0 ));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 0 < s ) vI *= randomBlotch( t+6.0+19.* float(0));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 1 < s ) vI *= randomBlotch( t+6.0+19.* float(1));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 2 < s ) vI *= randomBlotch( t+6.0+19.* float(2));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 3 < s ) vI *= randomBlotch( t+6.0+19.* float(3));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 4 < s ) vI *= randomBlotch( t+6.0+19.* float(4));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 5 < s ) vI *= randomBlotch( t+6.0+19.* float(5));"+#CRLF$
              System\Shader_Fragment_Text + "#endif"+#CRLF$
              System\Shader_Fragment_Text + "gl_FragColor.xyz = oldImage * vI;"+#CRLF$
              System\Shader_Fragment_Text + "#ifdef GRAIN"+#CRLF$
              System\Shader_Fragment_Text + "gl_FragColor.xyz *= (1.0+(rand(uv+t*.01)-.2)*.15);"+#CRLF$
              System\Shader_Fragment_Text + "#endif"+#CRLF$
              System\Shader_Fragment_Text + "}"+#CRLF$
          EndSelect
        CompilerCase #PB_OS_Windows
          Select Fragment
            Case 0
              System\Shader_Fragment_Text = "#version 120"+#CRLF$
              System\Shader_Fragment_Text + "//Normal-Image (Windows-Version)"+#CRLF$
              System\Shader_Fragment_Text + "uniform sampler2D color_texture;"+#CRLF$
              System\Shader_Fragment_Text + "uniform vec2 resolution;"+#CRLF$
              System\Shader_Fragment_Text + "void main() {"+#CRLF$
              System\Shader_Fragment_Text + "   vec2 pos = gl_FragCoord.xy/resolution;"+#CRLF$
              System\Shader_Fragment_Text + "   vec4 color = texture2D(color_texture, pos);"+#CRLF$
              System\Shader_Fragment_Text + "   gl_FragColor = color;"+#CRLF$
              System\Shader_Fragment_Text + "}"+#CRLF$
            Case 1
              System\Shader_Fragment_Text = "#version 120"+#CRLF$
              System\Shader_Fragment_Text + "//Gray-Image (Windows-Version)"+#CRLF$
              System\Shader_Fragment_Text + "uniform sampler2D color_texture;"+#CRLF$
              System\Shader_Fragment_Text + "uniform vec2 resolution;"+#CRLF$
              System\Shader_Fragment_Text + "void main() {"+#CRLF$
              System\Shader_Fragment_Text + "   vec2 pos = gl_FragCoord.xy/resolution;"+#CRLF$
              System\Shader_Fragment_Text + "   vec4 color = texture2D(color_texture, pos);"+#CRLF$
              System\Shader_Fragment_Text + "   float grey = (0.2126 * color.r) + (0.7152 * color.g) + (0.0722 * color.b);"+#CRLF$
              System\Shader_Fragment_Text + "   color.rgb += (grey - color.rgb);"+#CRLF$
              System\Shader_Fragment_Text + "   gl_FragColor = color;"+#CRLF$
              System\Shader_Fragment_Text + "}"+#CRLF$
            Case 2
              System\Shader_Fragment_Text = "#version 120"+#CRLF$
              System\Shader_Fragment_Text + "//Sepia-Image (Windows-Version)"+#CRLF$
              System\Shader_Fragment_Text + "uniform sampler2D color_texture;"+#CRLF$
              System\Shader_Fragment_Text + "uniform vec2 resolution;"+#CRLF$
              System\Shader_Fragment_Text + "void main() {"+#CRLF$
              System\Shader_Fragment_Text + "   float amount = 1.0;"+#CRLF$
              System\Shader_Fragment_Text + "   vec2 pos = gl_FragCoord.xy/resolution;"+#CRLF$
              System\Shader_Fragment_Text + "   vec4 color = texture2D(color_texture, pos);"+#CRLF$
              System\Shader_Fragment_Text + "   float r = color.r;"+#CRLF$
              System\Shader_Fragment_Text + "   float g = color.g;"+#CRLF$
              System\Shader_Fragment_Text + "   float b = color.b;"+#CRLF$
              System\Shader_Fragment_Text + "   color.r = min(1.0, (r * (1.0 - (0.607 * amount))) + (g * (0.769 * amount)) + (b * (0.189 * amount)));"+#CRLF$
              System\Shader_Fragment_Text + "   color.g = min(1.0, (r * 0.349 * amount) + (g * (1.0 - (0.314 * amount))) + (b * 0.168 * amount));"+#CRLF$
              System\Shader_Fragment_Text + "   color.b = min(1.0, (r * 0.272 * amount) + (g * 0.534 * amount) + (b * (1.0 - (0.869 * amount))));"+#CRLF$
              System\Shader_Fragment_Text + "   gl_FragColor = color;"+#CRLF$
              System\Shader_Fragment_Text + "}"+#CRLF$
            Case 3
              System\Shader_Fragment_Text = "#version 130"+#CRLF$
              System\Shader_Fragment_Text + "//Invert-Image (Windows-Version)"+#CRLF$
              System\Shader_Fragment_Text + "uniform sampler2D color_texture;"+#CRLF$
              System\Shader_Fragment_Text + "uniform vec2 resolution;"+#CRLF$
              System\Shader_Fragment_Text + "void main() {"+#CRLF$
              System\Shader_Fragment_Text + "   float amount = 1.0;"+#CRLF$
              System\Shader_Fragment_Text + "   vec2 pos = gl_FragCoord.xy/resolution;"+#CRLF$
              System\Shader_Fragment_Text + "   vec4 color = texture2D(color_texture, pos);"+#CRLF$
              System\Shader_Fragment_Text + "   gl_FragColor = texture2D(color_texture, pos);"+#CRLF$
              System\Shader_Fragment_Text + "   gl_FragColor.rgb = 1.0 - gl_FragColor.rgb;"+#CRLF$
              System\Shader_Fragment_Text + "}"+#CRLF$
            Case 4
              System\Shader_Fragment_Text = "#version 130"+#CRLF$
              System\Shader_Fragment_Text + "//Old-Movie (Windows-Version)"+#CRLF$
              System\Shader_Fragment_Text + "//#define BLACK_AND_WHITE"+#CRLF$ ;Einfach mal in Farbe ?
              System\Shader_Fragment_Text + "#define LINES_AND_FLICKER"+#CRLF$
              System\Shader_Fragment_Text + "#define BLOTCHES"+#CRLF$
              System\Shader_Fragment_Text + "#define GRAIN"+#CRLF$
              System\Shader_Fragment_Text + "#define FREQUENCY 15.0"+#CRLF$
              System\Shader_Fragment_Text + "uniform sampler2D color_texture;"+#CRLF$
              System\Shader_Fragment_Text + "uniform vec2 resolution;"+#CRLF$
              System\Shader_Fragment_Text + "uniform float time;"+#CRLF$
              System\Shader_Fragment_Text + "vec2 uv;"+#CRLF$
              System\Shader_Fragment_Text + "float rand(vec2 co){"+#CRLF$
              System\Shader_Fragment_Text + "return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);"+#CRLF$
              System\Shader_Fragment_Text + "}"+#CRLF$
              System\Shader_Fragment_Text + "float rand(float c){"+#CRLF$
              System\Shader_Fragment_Text + "return rand(vec2(c,1.0));"+#CRLF$
              System\Shader_Fragment_Text + "}"+#CRLF$
              System\Shader_Fragment_Text + "float randomLine(float seed){"+#CRLF$
              System\Shader_Fragment_Text + "float b = 0.01 * rand(seed);"+#CRLF$
              System\Shader_Fragment_Text + "float a = rand(seed+1.0);"+#CRLF$
              System\Shader_Fragment_Text + "float c = rand(seed+2.0) - 0.5;"+#CRLF$
              System\Shader_Fragment_Text + "float mu = rand(seed+3.0);"+#CRLF$
              System\Shader_Fragment_Text + "float l = 1.0;"+#CRLF$
              System\Shader_Fragment_Text + "if ( mu > 0.2)"+#CRLF$
              System\Shader_Fragment_Text + "l = pow(  abs(a * uv.x + b * uv.y + c ), 1.0/8.0 );"+#CRLF$
              System\Shader_Fragment_Text + "else"+#CRLF$
              System\Shader_Fragment_Text + "l = 2.0 - pow( abs(a * uv.x + b * uv.y + c), 1.0/8.0 );"+#CRLF$
              System\Shader_Fragment_Text + "return mix(0.5, 1.0, l);"+#CRLF$
              System\Shader_Fragment_Text + "}"+#CRLF$
              System\Shader_Fragment_Text + "float randomBlotch(float seed){"+#CRLF$
              System\Shader_Fragment_Text + "float x = rand(seed);"+#CRLF$
              System\Shader_Fragment_Text + "float y = rand(seed+1.0);"+#CRLF$
              System\Shader_Fragment_Text + "float s = 0.01 * rand(seed+2.0);"+#CRLF$
              System\Shader_Fragment_Text + "vec2 p = vec2(x,y) - uv;"+#CRLF$
              System\Shader_Fragment_Text + "p.x *= resolution.x / resolution.y;"+#CRLF$
              System\Shader_Fragment_Text + "float a = atan(p.y,p.x);"+#CRLF$
              System\Shader_Fragment_Text + "float v = 1.0;"+#CRLF$
              System\Shader_Fragment_Text + "float ss = s*s * (sin(6.2831*a*x)*0.1 + 1.0);"+#CRLF$
              System\Shader_Fragment_Text + "if ( dot(p,p) < ss ) v = 0.2;"+#CRLF$
              System\Shader_Fragment_Text + "else"+#CRLF$
              System\Shader_Fragment_Text + "v = pow(dot(p,p) - ss, 1.0/16.0);"+#CRLF$
              System\Shader_Fragment_Text + "return mix(0.3 + 0.2 * (1.0 - (s / 0.02)), 1.0, v);"+#CRLF$
              System\Shader_Fragment_Text + "}"+#CRLF$
              System\Shader_Fragment_Text + "void main(void){"+#CRLF$
              System\Shader_Fragment_Text + "uv = gl_FragCoord.xy / resolution.xy;"+#CRLF$
              System\Shader_Fragment_Text + "float t = float(int(time * FREQUENCY));"+#CRLF$
              System\Shader_Fragment_Text + "vec2 suv = uv + 0.002 * vec2( rand(t), rand(t + 23.0));"+#CRLF$
              System\Shader_Fragment_Text + "vec3 image = texture2D( color_texture, vec2(suv.x, suv.y) ).xyz;"+#CRLF$
              System\Shader_Fragment_Text + "#ifdef BLACK_AND_WHITE"+#CRLF$
              System\Shader_Fragment_Text + "float luma = dot( vec3(0.2126, 0.7152, 0.0722), image );"+#CRLF$
              System\Shader_Fragment_Text + "vec3 oldImage = luma * vec3(0.7, 0.7, 0.7);"+#CRLF$
              System\Shader_Fragment_Text + "#else"+#CRLF$
              System\Shader_Fragment_Text + "vec3 oldImage = image;"+#CRLF$
              System\Shader_Fragment_Text + "#endif"+#CRLF$
              System\Shader_Fragment_Text + "float vI = 16.0 * (uv.x * (1.0-uv.x) * uv.y * (1.0-uv.y));"+#CRLF$
              System\Shader_Fragment_Text + "vI *= mix( 0.7, 1.0, rand(t + 0.5));"+#CRLF$
              System\Shader_Fragment_Text + "vI += 1.0 + 0.4 * rand(t+8.);"+#CRLF$
              System\Shader_Fragment_Text + "vI *= pow(16.0 * uv.x * (1.0-uv.x) * uv.y * (1.0-uv.y), 0.4);"+#CRLF$
              System\Shader_Fragment_Text + "#ifdef LINES_AND_FLICKER"+#CRLF$
              System\Shader_Fragment_Text + "int l = int(8.0 * rand(t+7.0));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 0 < l ) vI *= randomLine( t+6.0+17.* float(0));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 1 < l ) vI *= randomLine( t+6.0+17.* float(1));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 2 < l ) vI *= randomLine( t+6.0+17.* float(2));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 3 < l ) vI *= randomLine( t+6.0+17.* float(3));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 4 < l ) vI *= randomLine( t+6.0+17.* float(4));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 5 < l ) vI *= randomLine( t+6.0+17.* float(5));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 6 < l ) vI *= randomLine( t+6.0+17.* float(6));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 7 < l ) vI *= randomLine( t+6.0+17.* float(7));"+#CRLF$
              System\Shader_Fragment_Text + "#endif"+#CRLF$
              System\Shader_Fragment_Text + "#ifdef BLOTCHES"+#CRLF$
              System\Shader_Fragment_Text + "int s = int( max(8.0 * rand(t+18.0) -2.0, 0.0 ));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 0 < s ) vI *= randomBlotch( t+6.0+19.* float(0));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 1 < s ) vI *= randomBlotch( t+6.0+19.* float(1));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 2 < s ) vI *= randomBlotch( t+6.0+19.* float(2));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 3 < s ) vI *= randomBlotch( t+6.0+19.* float(3));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 4 < s ) vI *= randomBlotch( t+6.0+19.* float(4));"+#CRLF$
              System\Shader_Fragment_Text + "if ( 5 < s ) vI *= randomBlotch( t+6.0+19.* float(5));"+#CRLF$
              System\Shader_Fragment_Text + "#endif"+#CRLF$
              System\Shader_Fragment_Text + "gl_FragColor.xyz = oldImage * vI;"+#CRLF$
              System\Shader_Fragment_Text + "#ifdef GRAIN"+#CRLF$
              System\Shader_Fragment_Text + "gl_FragColor.xyz *= (1.0+(rand(uv+t*.01)-.2)*.15);"+#CRLF$
              System\Shader_Fragment_Text + "#endif"+#CRLF$
              System\Shader_Fragment_Text + "}"+#CRLF$
          EndSelect
      CompilerEndSelect
      
      System\Shader_Program = Shader_Compile_Link_Use(System\Shader_Vertex_Text, System\Shader_Fragment_Text)
      
      If System\Shader_Program = 0
        MessageRequester("Error?","Kein Shader verfügbar!",#PB_MessageRequester_Error)
        End
      EndIf
      
      ;Shader-Variablen-Vermittlung!
      
      System\Shader_Uniform_Time = glGetUniformLocation(System\Shader_Program, "time")
      ;System\Shader_Uniform_Mouse = glGetUniformLocation(System\Shader_Program, "mouse")
      System\Shader_Uniform_Resolution = glGetUniformLocation(System\Shader_Program, "resolution")
      System\Shader_Uniform_SurfacePosition.i = glGetUniformLocation(System\Shader_Program, "surfacePosition")
      
      System\Shader_Image = glGetUniformLocation(System\Shader_Program, "color_texture")
      
      glBindTexture_(#GL_TEXTURE_2D, TextureID)
      glUniform1i(System\Shader_Image, 0)
      
      ProcedureReturn #True
    Else
      MessageRequester("Error?","Der ausgewählte Filter existiert nicht!",#PB_MessageRequester_Error)
    EndIf
  EndProcedure
  
  Procedure Render_ScreenFilter(w.i, h.i, ftime.i)
    If System\Shader_Program <> 0
      System\App_CurrentTime = ftime
      System\Shader_Width = w
      System\Shader_Height = h
      
      glEnable_(#GL_TEXTURE_2D)
      glMatrixMode_(#GL_TEXTURE_2D)
      glLoadIdentity_()
      
      glUniform2f(System\Shader_Uniform_Resolution,System\Shader_Width,System\Shader_Height)
      glUniform1f(System\Shader_Uniform_Time,(System\App_CurrentTime-System\App_StartTime) / 1000.0)
      glUniform2i(System\Shader_Uniform_SurfacePosition.i,1.0,1.0)
      glUniform1i(glGetUniformLocation(System\Shader_Program, "color_texture"), #GL_TEXTURE)
      
      CompilerSelect #PB_Compiler_OS
        CompilerCase #PB_OS_Linux
          glBegin_(#GL_QUADS)
          glTexCoord2i_(0, h) : glVertex2f_(-1, -1)
          glTexCoord2i_(w, h) : glVertex2f_(1, -1)
          glTexCoord2i_(w, 0) : glVertex2f_(1, 1)
          glTexCoord2i_(0, 0) : glVertex2f_(-1, 1)
          glEnd_()
        CompilerCase #PB_OS_Windows
          glBegin_(#GL_QUADS)
          glTexCoord2i_(0, 0) : glVertex2f_(-1, -1)
          glTexCoord2i_(w, 0) : glVertex2f_(1, -1)
          glTexCoord2i_(w, h) : glVertex2f_(1, 1)
          glTexCoord2i_(0, h) : glVertex2f_(-1, 1)
          glEnd_()
      CompilerEndSelect
      
      glDisable_(#GL_TEXTURE_2D)
    EndIf
  EndProcedure
  
EndModule
; IDE Options = PureBasic 5.70 LTS beta 2 (Windows - x64)
; CursorPosition = 199
; FirstLine = 195
; Folding = ---
; EnableThread
; EnableXP
; DPIAware
; EnableOnError
; SubSystem = opengl
; Compiler = PureBasic 5.70 LTS beta 2 (Windows - x64)